@@ -59,17 +59,17 @@ class BufferingBase(object):
                     break
                 try:
                     data = data.decode(self.ENCODING)
-                except Exception:
-                    logger.exception("Failed to decode %r" % data)
+                except Exception as exc:
+                    logger.exception("%r failed to decode %r: %r", self, data, exc)
                     raise
 
                 self.buff.write(data)
-        except OSError as e:
-            if e.errno not in (
+        except OSError as exc:
+            if exc.errno not in (
                     errno.EAGAIN, errno.EWOULDBLOCK,
                     errno.EINPROGRESS
             ):
-                print("Failed to read from %s: %s" % (self.fd, e))
+                logger.exception("%r failed to read from FD %s: %r", self, self.fd, exc)
         return self.buff.getvalue()
 
     def reset(self):
@@ -95,8 +95,8 @@ class ThreadedBufferingBase(BufferingBase):
                     self.queue.put(data)
                 else:
                     time.sleep(1)
-            except OSError as e:
-                print("Failed to read from %s: %s" % (self.fh, e))
+            except OSError as exc:
+                logger.exception("%r failed to read from %s: %r", self, self.fh, exc)
                 raise
 
     def read(self):
@@ -107,8 +107,8 @@ class ThreadedBufferingBase(BufferingBase):
                 break
             try:
                 data = data.decode(self.ENCODING)
-            except Exception:
-                logger.exception("Failed to decode %r" % data)
+            except Exception as exc:
+                logger.exception("%r failed to decode %r: %r", self, data, exc)
                 raise
             self.buff.write(data)
         return self.buff.getvalue()
@@ -118,6 +118,8 @@ class ThreadedBufferingBase(BufferingBase):
 
 
 class TestProcess(BufferingBase if fcntl else ThreadedBufferingBase):
+    __test__ = False
+
     def __init__(self, *args, **kwargs):
         kwargs.setdefault('env', os.environ)
         kwargs.setdefault('bufsize', 1)
@@ -125,6 +127,7 @@ class TestProcess(BufferingBase if fcntl else ThreadedBufferingBase):
             args,
             stdout=subprocess.PIPE,
             stderr=subprocess.STDOUT,
+            close_fds=True,
             **kwargs
         )
         super(TestProcess, self).__init__(self.proc.stdout)
@@ -136,6 +139,9 @@ class TestProcess(BufferingBase if fcntl else ThreadedBufferingBase):
     def signal(self, sig):
         self.proc.send_signal(sig)
 
+    def __repr__(self):
+        return "TestProcess(pid=%s, is_alive=%s)" % (self.proc.pid, self.is_alive)
+
     def __enter__(self):
         return self
 
@@ -153,12 +159,12 @@ class TestProcess(BufferingBase if fcntl else ThreadedBufferingBase):
                         if exc.errno == errno.ESRCH:
                             return
                         else:
-                            print("Failed to terminate %s: %s" % (self.proc.pid, exc))
+                            logger.exception("%r failed to terminate process: %r", self, exc)
                 else:
                     return
                 time.sleep(0.2)
             try:
-                print('Killing %s !' % self, file=sys.stderr)
+                logger.critical('%s killing unresponsive process!', self)
                 self.proc.kill()
             except OSError as exc:
                 if exc.errno != errno.ESRCH:
@@ -168,27 +174,25 @@ class TestProcess(BufferingBase if fcntl else ThreadedBufferingBase):
                 data, _ = self.proc.communicate()
                 try:
                     data = data.decode(self.ENCODING)
-                except Exception:
-                    logger.exception("Failed to decode %r" % data)
+                except Exception as exc:
+                    logger.exception("%s failed to decode %r: %r", self, data, exc)
                     raise
                 self.buff.write(data)
             except IOError as exc:
                 if exc.errno != errno.EAGAIN:
-                    print('\nFailed to cleanup process:\n', file=sys.stderr)
-                    traceback.print_exc()
-            except Exception:
-                print('\nFailed to cleanup process:\n', file=sys.stderr)
-                traceback.print_exc()
+                    logger.exception('%s failed to cleanup buffers: %r', self, exc)
+            except Exception as exc:
+                logger.exception('%s failed to cleanup buffers: %r', self, exc)
             try:
                 self.cleanup()
-            except Exception:
-                print('\nFailed to cleanup process:\n', file=sys.stderr)
-                traceback.print_exc()
+            except Exception as exc:
+                logger.exception('%s failed to cleanup: %r', self, exc)
 
     close = __exit__
 
 
 class TestSocket(BufferingBase if fcntl else ThreadedBufferingBase):
+    __test__ = False
     BUFFSIZE = 8192
 
     def __init__(self, sock):
@@ -222,8 +226,6 @@ def wait_for_strings(cb, seconds, *strings):
     """
     This checks that *string appear in cb(), IN THE GIVEN ORDER !
     """
-    buff = '<UNINITIALIZED>'
-
     start = time.time()
     while True:
         buff = cb()
@@ -270,50 +272,3 @@ class ProcessTestCase(unittest.TestCase):
     dump_on_error = staticmethod(dump_on_error)
     wait_for_strings = staticmethod(wait_for_strings)
 
-
-_cov = None
-
-
-def restart_coverage():
-    logger.critical("(RE)STARTING COVERAGE.")
-    global _cov
-    try:
-        from coverage.control import coverage
-        from coverage.collector import Collector
-    except ImportError:
-        _cov = None
-        return
-    if _cov:
-        _cov.save()
-        _cov.stop()
-    if Collector._collectors:
-        Collector._collectors[-1].stop()
-
-    _cov = coverage(auto_data=True, data_suffix=True)
-    _cov.start()
-
-
-def setup_coverage(env_var="WITH_COVERAGE"):
-    """
-    Patch fork and forkpty to restart coverage measurement after fork. Expects to have a environment variable named WITH_COVERAGE set to a
-    non-empty value.
-    """
-    if os.environ.get(env_var) == 'yes':  # don't even bother if not set
-        restart_coverage()
-
-        handler = signal.getsignal(signal.SIGTERM)
-        if handler is None:
-            logger.critical("SIGTERM handler not installed from python. It's getting overriden !")
-            print("SIGTERM handler not installed from python. It's getting overriden !", file=sys.__stderr__)
-
-        @partial(signal.signal, signal.SIGTERM)
-        def on_atexit(signo, frame):
-            if _cov:
-                _cov._atexit()
-            if handler == signal.SIG_DFL:
-                signal.signal(signo, signal.SIG_DFL)
-                os.kill(os.getpid(), signo)
-            elif handler == signal.SIG_IGN:
-                pass
-            elif handler:
-                handler(signo, frame)
